#ifndef _MMC_DRIVER_H
#define _MMC_DRIVER_H

// Command list for MMC operation
#define CMD_GO_IDLE_STATE		0 	// Reset the MMC
#define CMD_SEND_OP_COND		1 	// Activate initialization process
#define CMD_SET_BLOCKLEN		16 	// Set block length (in bytes)
#define CMD_READ_BLOCK 			17 	// Read a block of data
#define CMD_WRITE_BLOCK 		24 	// Write a block of data
#define CMD_CRC_ONOFF 			59 	// 0 to turn off CRC, 1 to turn on CRC

// R1 type response bit defintion
#define R1_IN_IDLE_STATE		0x1

// Data Token
#define DATA_TOKEN 				0xFE	// First byte sent when data is read and written


#include "type.h"
#include <Cdef21369.h>
#include <def21369.h>
#include <stdio.h>
#include <signal.h>
#include <sru.h>



// Function Declarations
void 	read_SectorData(WORD sublocation, BYTE *buffer, WORD size);
void 	write_SectorData(WORD sublocation, BYTE *buffer, WORD size);
void 	read_BufferSector(BYTE* buff,DWORD sector_offset,int count);
void 	write_BufferSector(BYTE* buff,DWORD sector_offset,int count);
BYTE 	read_BYTE(WORD sublocation);
WORD 	read_WORD(WORD sublocation);
DWORD 	read_DWORD(WORD sublocation); 
void 	write_BYTE(WORD sublocation, BYTE value);
void 	write_WORD(WORD sublocation, WORD value);



// Function Declarations
// MMC Functions
void 	mmc_init (void);
void 	mmc_read_data (int address, int bytes, int* destination);
void 	mmc_write_data (int address, int* source);
void 	mmc_send_cmd(int cmd, int arg);
int 	mmc_get_response(void);
void 	mmc_set_block_length(int length);
void 	init_spi(void);
void 	mmc_crc_on_off(int on_off);

// Interrupt Functions
void 	mmc_init_isr(int sig);
void 	wait_for_token(int sig);
void 	command_isr(int sig);
void 	read_block_isr(int sig);
void 	write_block_isr(int sig);
void 	wait_for_data_response(int sig);


typedef union 
{
	unsigned char 	u8data[512];
	unsigned short 	u16data[256];
	unsigned long 	u32data[128];
} BUFFER;

typedef struct
{
 	BUFFER currentsector;
	DWORD SectorCurrentlyLoaded; 
} MMC_BUFFER;

MMC_BUFFER buffers;

BYTE read_buff[512];
BYTE write_buff[512];

#endif
